/*
 * Copyright (C) 2006-2007 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

/** \file recv_common.c common functions of para_recv and para_audiod */

#include "para.h"

#include "list.h"
#include "sched.h"
#include "recv.h"
#include "string.h"

DEFINE_RECEIVER_ARRAY;
static void *parse_receiver_args(int receiver_num, char *options)
{
	struct receiver *r = &receivers[receiver_num];
	char **argv;
	int argc, i;
	void *conf;


//	PARA_DEBUG_LOG("%s, options: %s\n", r->name,
//		options? options : "(none)");
	if (options) {
		PARA_DEBUG_LOG("options: %s\n", options);
		argc = split_args(options, &argv, " \t");
		for (i = argc - 1; i >= 0; i--)
			argv[i + 1] = argv[i];
		argv[0] = para_strdup(r->name);
		argc += 1;
		PARA_DEBUG_LOG("argc = %d, argv[0]: %s\n", argc, argv[0]);
	} else {
		argc = 1;
		argv = para_malloc(2 * sizeof(char*));
		argv[0] = NULL;
		argv[1] = NULL;
	}
	conf = r->parse_config(argc, argv);
	free(argv[0]);
	free(argv);
	return conf;
}

/**
 * check if given string is a valid command line for any receiver
 *
 * \param \ra string of the form receiver_name:options
 * \param receiver_num contains the number of the receiver upon success
 *
 * This function checks whether \a ra starts with the name of a supported
 * paraslash receiver, optinally followed by a colon and any options for that
 * receiver. If a valid receiver name was found and further are present, the
 * remaining part of \a ra is passed to that receiver's config parser.
 *
 * \return On success, a pointer to the gengetopt args info struct is returned
 * and \a receiver_num contains the number of the receiver. Otherwise this function
 * returns \p NULL.
 */
void *check_receiver_arg(char *ra, int *receiver_num)
{
	int j;

	PARA_DEBUG_LOG("checking %s\n", ra);
	for (j = 0; receivers[j].name; j++) {
		const char *name = receivers[j].name;
		size_t len = strlen(name);
		char c;
		if (strlen(ra) < len)
			continue;
		if (strncmp(name, ra, len))
			continue;
		c = ra[len];
		if (c && c != ' ')
			continue;
		if (c && !receivers[j].parse_config)
			return NULL;
		*receiver_num = j;
		return parse_receiver_args(j, c? ra + len + 1: NULL);
	}
	PARA_ERROR_LOG("%s", "receiver not found\n");
	return NULL;
}
